path = require 'path'
express = require 'express'
derby = require 'derby'
gzip = require 'connect-gzip'
sink = require './sink'


## SERVER CONFIGURATION ##

MAX_AGE_ONE_YEAR = maxAge: 1000 * 60 * 60 * 24 * 365
root = __dirname
publicPath = path.join root, 'public'
staticPages = derby.createStatic root

(server = express.createServer())
  # The express.static middleware can be used instead of gzip.staticGzip
  .use(gzip.staticGzip publicPath, MAX_AGE_ONE_YEAR)
  .use(express.favicon())

  # Form data parsing support
  .use(express.bodyParser())
  .use(express.methodOverride())

  # Remove to disable dynamic gzipping
  .use(gzip.gzip())

  # The router method creates an express middleware from the app's routes
  .use(sink.router())
  .use(server.router)


## ERROR HANDLING ##

server.configure 'development', ->
  # Log errors in development only
  server.error (err, req, res, next) ->
    if err then console.log(if err.stack then err.stack else err)
    next err

server.error (err, req, res) ->
  ## Customize error handling here ##
  message = err.message || err.toString()
  status = parseInt message
  res.send if 400 <= status < 600 then status else 500


## SERVER ONLY ROUTES ##

server.all '*', (req) ->
  throw "404: #{req.url}"


## STORE SETUP ##

store = sink.createStore redis: {db: 1}, listen: server

## TODO: Remove when using a database ##
# Clear all data every time the node server is started
store.flush()

server.listen 3001
console.log 'Express server started in %s mode', server.settings.env
console.log 'Go to: http://localhost:%d/', server.address().port
