var ctxFor, get, pages, ready, sink, view, _ref;

_ref = sink = require('derby').createApp(module), get = _ref.get, view = _ref.view, ready = _ref.ready;

pages = [
  {
    name: 'home',
    text: 'Home',
    url: '/'
  }, {
    name: 'liveCss',
    text: 'Live CSS',
    url: '/live-css'
  }, {
    name: 'tableEditor',
    text: 'Table editor',
    url: '/table'
  }, {
    name: 'submit',
    text: 'Submit form',
    url: '/submit'
  }, {
    name: 'back',
    text: 'Back redirect',
    url: '/back'
  }, {
    name: 'error',
    text: 'Error test',
    url: '/error'
  }
];

ctxFor = function(name, ctx) {
  var i, last, page;
  if (ctx == null) ctx = {};
  ctx[name + 'Visible'] = true;
  last = pages.length - 1;
  ctx.pages = (function() {
    var _len, _results;
    _results = [];
    for (i = 0, _len = pages.length; i < _len; i++) {
      page = pages[i];
      page = Object.create(page);
      if (page.name === name) {
        page.current = true;
        ctx.title = page.text;
      }
      page.last = i === last;
      _results.push(page);
    }
    return _results;
  })();
  return ctx;
};

get('/', function(page) {
  return page.render(ctxFor('home'));
});

get('/live-css', function(page, model) {
  return model.subscribe('liveCss', function() {
    model.setNull('liveCss.styles', [
      {
        prop: 'color',
        value: '#c00',
        active: true
      }, {
        prop: 'font-weight',
        value: 'bold',
        active: true
      }, {
        prop: 'font-size',
        value: '18px',
        active: false
      }
    ]);
    model.setNull('liveCss.outputText', 'Edit this text...');
    return page.render(ctxFor('liveCss'));
  });
});

get('/table', function(page, model) {
  return model.subscribe('rows', 'cols', function() {
    if (!model.get('rows')) {
      model.set('rows', [
        {
          name: 'A',
          cells: [{}, {}, {}]
        }, {
          name: 'B',
          cells: [{}, {}, {}]
        }
      ]);
      model.set('cols', [
        {
          name: 1
        }, {
          name: 2
        }, {
          name: 3
        }
      ]);
    }
    return page.render(ctxFor('tableEditor'));
  });
});

['get', 'post', 'put', 'del'].forEach(function(method) {
  return sink[method]('/submit', function(page, model, _arg) {
    var args, body, query;
    body = _arg.body, query = _arg.query;
    args = JSON.stringify({
      method: method,
      body: body,
      query: query
    }, null, '  ');
    return page.render(ctxFor('submit', {
      args: args
    }));
  });
});

get('/error', function() {
  throw new Error(500);
});

get('/back', function(page) {
  return page.redirect('back');
});

view.make('Title', 'Derby demo: {{title}}');

view.make('Head', '<style>\n  p{margin:0;padding:0}\n  body{margin:10px}\n  body,select{font:13px/normal arial,sans-serif}\n  .css{list-style:none;padding-left:1em;margin:0}\n</style>');

view.make('Body', '<p>\n{{#pages}}\n  {{#current}}\n    <b>{{text}}</b>{{^last}} | {{/}}\n  {{^}}\n    <a href={{url}}>{{text}}</a>{{^last}} | {{/}}\n  {{/}}\n{{/}}\n<hr>\n{{homeVisible > home}}\n{{liveCssVisible > liveCss}}\n{{tableEditorVisible > tableEditor}}\n{{submitVisible > submit}}');

view.make('home', '<h1>Welcome!</h1>\n<p>This is a collection of random demos. Check \'em out! <a href=#jump>Test jump link</a>\n<p style=margin-top:1000px>\n<a name=jump>Jumped!</a>');

view.make('liveCss', '<select multiple><optgroup label="CSS properties">\n  ((#liveCss.styles))<option selected=((.active))>((.prop))((/))\n</select>\n<div>\n  ((#liveCss.styles))\n    <p><input type=checkbox checked=((.active))> \n    <input value=((.prop)) disabled=!((.active))> \n    <input value=((.value)) disabled=!((.active))> \n    <button x-bind=click:deleteStyle>Delete</button>\n  ((/))\n</div>\n<button x-bind=click:addStyle>Add</button>\n<h3>Currently applied:</h3>\n<p>{\n<ul class=css>\n  ((#liveCss.styles :style))\n    <li x-displayed=((:style.active))>((> cssProperty))\n  ((/))\n</ul>\n<p>}\n<h3>Output:</h3>\n<p style="((liveCss.styles :style > cssProperty))" contenteditable>\n  (((liveCss.outputText)))\n</p>');

view.make('cssProperty', '((#:style.active))((:style.prop)): ((:style.value));((/))');

view.make('tableEditor', '<table>\n  <thead>\n    <tr>\n      <th></th>\n      ((#cols))<th>((.name))</th>((/))\n      <th></th>\n  <tbody>\n    ((#rows))\n      <tr>\n        <th>((.name))</th>\n        ((#.cells))<td><input value=((.text))></td>((/))\n        <td><button x-bind=click:deleteRow>Delete</button></td>\n      </tr>\n    ((/))\n  <tfoot>\n    <tr>\n      <th></th>\n      ((#cols))<th><button x-bind=click:deleteCol>Delete</button></th>((/))\n      <th></th>\n</table>');

view.make('submit', '<form action=submit method=post>\n  <input type=hidden name=_method value=put>\n  <p><label>Name: <input type=text name=user[name]></label>\n  <p><label>Email: <input type=text name=user[email]></label>\n  <p><input type=submit>\n</form>\n<h3>Arguments:</h3>\n<pre><code>{{args}}</code></pre>');

ready(function(model) {
  var targetIndex;
  exports.addStyle = function() {
    return model.push('liveCss.styles', {});
  };
  targetIndex = function(e, levels) {
    var child, i, item, _len, _ref2;
    if (levels == null) levels = 1;
    item = e.target;
    while (levels--) {
      item = item.parentNode;
    }
    _ref2 = item.parentNode.childNodes;
    for (i = 0, _len = _ref2.length; i < _len; i++) {
      child = _ref2[i];
      if (child === item) return i;
    }
  };
  exports.deleteStyle = function(e) {
    return model.remove('liveCss.styles', targetIndex(e));
  };
  exports.deleteRow = function(e) {
    return model.remove('rows', targetIndex(e, 2));
  };
  return exports.deleteCol = function(e) {
    var i, row;
    i = targetIndex(e) - 2;
    row = model.get('rows').length;
    while (row--) {
      model.remove("rows." + row + ".cells", i);
    }
    return model.remove('cols', i);
  };
});
