var Dom, EventDispatcher, addListener, body, clearElements, dist, distribute, doc, domHandler, element, elements, getMethods, getNaN, getProp, getRange, htmlEscape, inPage, markers, setMethods, win;

EventDispatcher = require('./EventDispatcher');

htmlEscape = require('./html').htmlEscape;

elements = markers = null;

if (win = typeof window !== 'undefined' && window) {
  doc = win.document;
  body = doc.body;
  if (!body.insertAdjacentHTML) {
    HTMLElement.prototype.insertAdjacentHTML = function(position, html) {
      var container, firstChild, nextSibling, node, parent, ref;
      ref = this;
      parent = ref.parentNode;
      container = doc.createElement(parent.tagName);
      container.innerHTML = html;
      switch (position.toLowerCase()) {
        case 'beforeend':
          while (node = container.firstChild) {
            ref.appendChild(node);
          }
          break;
        case 'beforebegin':
          while (node = container.firstChild) {
            parent.insertBefore(node, ref);
          }
          break;
        case 'afterend':
          nextSibling = ref.nextSibling;
          while (node = container.lastChild) {
            nextSibling = parent.insertBefore(node, nextSibling);
          }
          break;
        case 'afterbegin':
          firstChild = ref.firstChild;
          while (node = container.lastChild) {
            firstChild = ref.insertBefore(node, firstChild);
          }
      }
    };
  }
}

inPage = function(node) {
  return body.compareDocumentPosition(node) & 16;
};

getRange = function(name) {
  var comment, commentIterator, end, range, start;
  start = markers[name];
  end = markers['$' + name];
  if (!(start && end)) {
    commentIterator = doc.createNodeIterator(body, 128, null, false);
    while (comment = commentIterator.nextNode()) {
      markers[comment.data] = comment;
    }
    start = markers[name];
    end = markers['$' + name];
    if (!(start && end)) return;
  }
  if (!inPage(start)) {
    delete markers[name];
    delete markers['$' + name];
    return;
  }
  range = doc.createRange();
  range.setStartAfter(start);
  range.setEndBefore(end);
  return range;
};

element = function(id) {
  return elements[id] || (elements[id] = doc.getElementById(id)) || getRange(id);
};

clearElements = function() {
  var ranges;
  elements = {
    $win: win,
    $doc: doc
  };
  window.markers = markers = {};
  return ranges = {};
};

getNaN = function() {
  return NaN;
};

getMethods = {
  attr: function(el, attr) {
    return el.getAttribute(attr);
  },
  prop: getProp = function(el, prop) {
    return el[prop];
  },
  propPolite: getProp,
  html: function(el) {
    return el.innerHTML;
  },
  visible: getNaN,
  displayed: getNaN,
  append: getNaN,
  insert: getNaN,
  remove: getNaN,
  move: getNaN
};

setMethods = {
  attr: function(el, ignore, value, attr) {
    if (ignore && el.id === ignore) return;
    return el.setAttribute(attr, value);
  },
  prop: function(el, ignore, value, prop) {
    if (ignore && el.id === ignore) return;
    return el[prop] = value;
  },
  propPolite: function(el, ignore, value, prop) {
    if (ignore && el.id === ignore) return;
    if (el !== doc.activeElement) return el[prop] = value;
  },
  visible: function(el, ignore, value) {
    if (ignore && el.id === ignore) return;
    return el.style.visibility = value ? '' : 'hidden';
  },
  displayed: function(el, ignore, value) {
    if (ignore && el.id === ignore) return;
    return el.style.display = value ? '' : 'none';
  },
  html: function(obj, ignore, value, escape) {
    if (escape) value = htmlEscape(value);
    if (obj.nodeType) {
      if (ignore && obj.id === ignore) return;
      return obj.innerHTML = value;
    } else {
      obj.deleteContents();
      return obj.insertNode(obj.createContextualFragment(value));
    }
  },
  append: function(obj, ignore, value, escape) {
    var el, ref;
    if (escape) value = htmlEscape(value);
    if (obj.nodeType) {
      return obj.insertAdjacentHTML('beforeend', value);
    } else {
      el = obj.endContainer;
      ref = el.childNodes[obj.endOffset];
      return el.insertBefore(obj.createContextualFragment(value), ref);
    }
  },
  insert: function(obj, ignore, value, escape, index) {
    var el, ref;
    if (escape) value = htmlEscape(value);
    if (obj.nodeType) {
      if (ref = obj.childNodes[index]) {
        return ref.insertAdjacentHTML('beforebegin', value);
      } else {
        return obj.insertAdjacentHTML('beforeend', value);
      }
    } else {
      el = obj.startContainer;
      ref = el.childNodes[obj.startOffset + index];
      return el.insertBefore(obj.createContextualFragment(value), ref);
    }
  },
  remove: function(el, ignore, index) {
    if (!el.nodeType) {
      index += el.startOffset;
      el = el.startContainer;
    }
    return el.removeChild(el.childNodes[index]);
  },
  move: function(el, ignore, from, to) {
    var child, offset, ref, toEl;
    if (!el.nodeType) {
      offset = el.startOffset;
      from += offset;
      to += offset;
      el = el.startContainer;
    }
    child = el.childNodes[from];
    if (ignore && (toEl = el.childNodes[to]) && toEl.id === ignore || child.id === ignore) {
      return;
    }
    ref = el.childNodes[to > from ? to + 1 : to];
    return el.insertBefore(child, ref);
  }
};

addListener = domHandler = function() {};

dist = function(e) {
  var child, childEvent, _i, _len, _ref, _results;
  _ref = e.target.childNodes;
  _results = [];
  for (_i = 0, _len = _ref.length; _i < _len; _i++) {
    child = _ref[_i];
    if (child.nodeType !== 1) return;
    childEvent = Object.create(e);
    childEvent.target = child;
    domHandler(childEvent);
    _results.push(dist(childEvent));
  }
  return _results;
};

distribute = function(e) {
  var clone, key, value;
  clone = {};
  for (key in e) {
    value = e[key];
    clone[key] = value;
  }
  return dist(clone);
};

Dom = module.exports = function(model, appExports, history) {
  var events;
  this.history = history;
  this.events = events = new EventDispatcher({
    onBind: function(name, listener) {
      if (listener.length > 3) listener[0] = model.__pathMap.id(listener[0]);
      if (!(name in events._names)) addListener(doc, name);
      return name;
    },
    onTrigger: function(name, listener, targetId, e) {
      var callback, delay, el, finish, fn, id, invert, method, path, pathId, property;
      id = listener[1];
      if (id !== targetId) return;
      if (!(el = element(id))) return false;
      if (listener.length <= 3) {
        fn = listener[0], id = listener[1], delay = listener[2];
        callback = fn === '$dist' ? distribute : appExports[fn];
        if (!callback) return;
      } else {
        pathId = listener[0], id = listener[1], method = listener[2], property = listener[3], delay = listener[4], invert = listener[5];
        if (!(path = model.__pathMap.paths[pathId])) return false;
      }
      finish = function() {
        var value;
        value = getMethods[method](el, property);
        if (invert) value = !value;
        if (model.get(path) === value) return;
        return model.set(path, value);
      };
      if (delay != null) {
        setTimeout(callback || finish, delay, e);
      } else {
        (callback || finish)(e);
      }
    }
  });
};

Dom.prototype = {
  clear: function() {
    this.events.clear();
    return clearElements();
  },
  init: function() {
    var events, history, name;
    events = this.events;
    history = this.history;
    clearElements();
    domHandler = function(e) {
      var target;
      target = e.target;
      if (target.nodeType === 3) target = target.parentNode;
      return events.trigger(e.type, target.id, e);
    };
    if (doc.addEventListener) {
      this.addListener = addListener = function(el, name, cb, captures) {
        if (cb == null) cb = domHandler;
        if (captures == null) captures = false;
        return el.addEventListener(name, cb, captures);
      };
    } else if (doc.attachEvent) {
      this.addListener = addListener = function(el, name, cb) {
        if (cb == null) cb = domHandler;
        return el.attachEvent('on' + name, function() {
          event.target || (event.target = event.srcElement);
          return cb(event);
        });
      };
    }
    for (name in events._names) {
      addListener(doc, name);
    }
    addListener(doc, 'click', function(e) {
      if (e.target.href && !e.metaKey && e.which === 1) {
        return history._onClickLink(e);
      }
    });
    addListener(doc, 'submit', function(e) {
      if (e.target.tagName.toLowerCase() === 'form') {
        return history._onSubmitForm(e);
      }
    });
    return addListener(win, 'popstate', function(e) {
      return history._onPop(e);
    });
  },
  update: function(id, method, ignore, value, property, index) {
    var el;
    if (!(el = element(id))) return false;
    if (value === getMethods[method](el, property)) return;
    setMethods[method](el, ignore, value, property, index);
  }
};

Dom.getMethods = getMethods;

Dom.setMethods = setMethods;
