var View, addDomEvent, addId, attrEscape, bindEvents, bindEventsById, bindEventsByIdString, blockFn, dataValue, defaultCtx, empty, extend, extendCtx, extractPlaceholder, forAttr, hasKeys, htmlEscape, modelPath, notFound, parse, parseAttr, parseElement, parseHtml, parseMatch, parsePlaceholder, partialFn, pushChars, pushVar, pushVarFns, pushVarString, reduceStack, renderer, textFn, trim, unescapeEntities, wrapPost, _ref, _ref2;

hasKeys = require('racer').util.hasKeys;

_ref = require('./html'), parseHtml = _ref.parse, unescapeEntities = _ref.unescapeEntities, htmlEscape = _ref.htmlEscape, attrEscape = _ref.attrEscape;

_ref2 = require('./parser'), modelPath = _ref2.modelPath, parsePlaceholder = _ref2.parsePlaceholder, parseElement = _ref2.parseElement, parseAttr = _ref2.parseAttr, addDomEvent = _ref2.addDomEvent;

empty = function() {
  return '';
};

notFound = function(name) {
  throw new Error("Can't find view: " + name);
};

defaultCtx = {
  $depth: 0,
  $aliases: {},
  $paths: [],
  $i: []
};

View = module.exports = function() {
  var self;
  self = this;
  this._views = Object.create(this["default"]);
  this._inline = '';
  this._idCount = 0;
  this._uniqueId = function() {
    return '$' + (self._idCount++).toString(36);
  };
};

View.prototype = {
  "default": {
    doctype: function() {
      return '<!DOCTYPE html><meta charset=utf-8>';
    },
    title$s: function() {
      return 'Derby app';
    },
    head: empty,
    header: empty,
    body: empty,
    scripts: empty,
    tail: empty
  },
  make: function(name, template, isString) {
    var onBind, render, self;
    if (!isString) this.make(name + '$s', template, true);
    name = name.toLowerCase();
    self = this;
    render = function(ctx) {
      render = parse(self, template, isString, onBind);
      return render(ctx);
    };
    this._views[name] = function(ctx) {
      return render(ctx);
    };
    if (name === 'title$s') {
      return onBind = function(events, name) {
        return bindEvents(events, name, render, ['$doc', 'prop', 'title']);
      };
    }
  },
  _find: function(name) {
    return this._views[name] || notFound(name);
  },
  get: function(name, ctx) {
    return this._find(name)(ctx ? extend(ctx, defaultCtx) : Object.create(defaultCtx));
  },
  before: function(name, before) {
    var render;
    render = this._find(name);
    return this._views[name] = function(ctx, model, triggerPath) {
      before(ctx);
      return render(ctx, model, triggerPath);
    };
  },
  after: function(name, after) {
    var render;
    render = this._find(name);
    return this._views[name] = function(ctx, model, triggerPath) {
      setTimeout(after, 0, ctx);
      return render(ctx, model, triggerPath);
    };
  },
  inline: empty,
  render: function(model, ctx, silent) {
    var body, title;
    this.model = model;
    this._idCount = 0;
    this.model.__pathMap.clear();
    this.model.__events.clear();
    this.dom.clear();
    title = this.get('title$s', ctx);
    body = this.get('header', ctx) + this.get('body', ctx);
    if (silent) return;
    document.body.innerHTML = body;
    return document.title = title;
  },
  htmlEscape: htmlEscape,
  attrEscape: attrEscape
};

extend = function(parent, obj) {
  var key, out;
  out = Object.create(parent);
  if (!obj) return out;
  for (key in obj) {
    out[key] = obj[key];
  }
  return out;
};

bindEvents = function(events, name, fn, params) {
  return events.push(function(ctx, modelEvents, domEvents, triggerId) {
    var listener, path;
    if (!(path = modelPath(ctx, name))) return;
    modelEvents.bind(path, listener = params.call ? params(triggerId) : params);
    listener.fn = fn;
    return listener.ctx = ctx.$stringCtx || ctx;
  });
};

bindEventsById = function(events, name, fn, attrs, method, prop) {
  return bindEvents(events, name, fn, function() {
    return [attrs._id || attrs.id, method, prop];
  });
};

bindEventsByIdString = function(events, name, fn, attrs, method, prop) {
  return bindEvents(events, name, fn, function(triggerId) {
    return [triggerId || attrs._id || attrs.id, method, prop];
  });
};

addId = function(view, attrs) {
  if (attrs.id == null) {
    return attrs.id = function() {
      return attrs._id = view._uniqueId();
    };
  }
};

View.trim = trim = function(s) {
  if (s) {
    return s.replace(/\n\s*/g, '');
  } else {
    return '';
  }
};

extractPlaceholder = function(text) {
  var content, match, _ref3;
  match = /^([^\{\(]*)(\{{2,3}|\({2,3})([^\}\)]+)(?:\}{2,3}|\){2,3})([\s\S]*)/.exec(text);
  if (!match) return;
  content = /^\s*([\#^\/]?)\s*([^\s>]*)(?:\s+:([^\s>]+))?(?:\s*>\s*([^\s]+)\s*)?/.exec(match[3]);
  return {
    pre: trim(match[1]),
    escaped: match[2].length === 2,
    bound: match[2].charAt(0) === '(',
    type: content[1],
    name: content[2],
    alias: content[3],
    partial: (_ref3 = content[4]) != null ? _ref3.toLowerCase() : void 0,
    post: trim(match[4])
  };
};

wrapPost = function(post) {
  if (!post) return false;
  return !/^(?:\{{2,3}|\({2,3})[\/^](?:\}{2,3}|\){2,3})/.test(post);
};

dataValue = function(ctx, model, name) {
  var path, value;
  if (path = modelPath(ctx, name)) {
    if ((value = model.get(path)) != null) {
      return value;
    } else {
      return model[path];
    }
  } else {
    return ctx[name];
  }
};

reduceStack = function(stack) {
  var attrs, bool, html, i, item, key, pushValue, value, _i, _len;
  html = [''];
  i = 0;
  for (_i = 0, _len = stack.length; _i < _len; _i++) {
    item = stack[_i];
    pushValue = function(value, isAttr) {
      if (value && value.call) {
        return i = html.push(value, '') - 1;
      } else {
        return html[i] += isAttr ? attrEscape(value) : value;
      }
    };
    switch (item[0]) {
      case 'start':
        html[i] += '<' + item[1];
        attrs = item[2];
        if ('id' in attrs) {
          html[i] += ' id=';
          pushValue(attrs.id, true);
        }
        for (key in attrs) {
          value = attrs[key];
          if (key === 'id') continue;
          if (value != null) {
            if (bool = value.bool) {
              pushValue(bool);
              continue;
            }
            html[i] += ' ' + key + '=';
            pushValue(value, true);
          } else {
            html[i] += ' ' + key;
          }
        }
        html[i] += '>';
        break;
      case 'chars':
        pushValue(item[1]);
        break;
      case 'end':
        html[i] += '</' + item[1] + '>';
        break;
      case 'marker':
        html[i] += '<!--' + item[1];
        pushValue(item[2].id);
        html[i] += '-->';
    }
  }
  return html;
};

renderer = function(view, items, events) {
  return function(ctx, model, triggerPath, triggerId) {
    var domEvents, event, html, i, index, indices, item, modelEvents, n, path, segment, triggerSegment, _i, _len, _len2, _path, _triggerPath;
    if (triggerPath && (path = ctx.$paths[0])) {
      _path = path.split('.');
      _triggerPath = triggerPath.split('.');
      indices = ctx.$i.slice();
      index = 0;
      for (i = 0, _len = _path.length; i < _len; i++) {
        segment = _path[i];
        triggerSegment = _triggerPath[i];
        if (segment === '$#' && (n = +triggerSegment) === n) {
          indices[index++] = n;
        } else if (segment !== triggerSegment) {
          break;
        }
      }
      ctx.$i = indices;
    }
    model = view.model;
    modelEvents = model.__events;
    domEvents = view.dom.events;
    html = '';
    for (_i = 0, _len2 = items.length; _i < _len2; _i++) {
      item = items[_i];
      html += item.call ? item(ctx, model, triggerPath) || '' : item;
    }
    i = 0;
    while (event = events[i++]) {
      event(ctx, modelEvents, domEvents, triggerId);
    }
    return html;
  };
};

parseMatch = function(view, match, queues, _arg) {
  var block, endBlock, endQueue, name, onEnd, onStart, onVar, queue, startBlock, type;
  onStart = _arg.onStart, onEnd = _arg.onEnd, onVar = _arg.onVar;
  type = match.type, name = match.name;
  block = queues.last().block;
  if (type === '/') {
    endBlock = true;
  } else if (type === '^') {
    startBlock = true;
    if (block && block.type === '#' && (!name || name === block.name)) {
      endBlock = true;
    }
  } else if (type === '#') {
    startBlock = true;
  }
  if (endBlock) {
    if (!block) throw new Error('Unmatched template end tag');
    match.name = block.name;
    endQueue = queues.pop();
  }
  if (startBlock) {
    queues.push(queue = {
      stack: [],
      events: [],
      block: match
    });
    if (endBlock) queue.closed = endQueue;
    onStart(queue);
  } else {
    if (endBlock) {
      onStart(queues.last());
      onEnd(endQueue);
    } else {
      onVar(match);
    }
  }
};

pushChars = function(stack, text) {
  if (text) return stack.push(['chars', text]);
};

extendCtx = function(ctx, value, name, alias, index, isArray) {
  var aliases, path;
  ctx = extend(ctx, value);
  if (path = modelPath(ctx, name, true)) ctx.$paths = [path].concat(ctx.$paths);
  if (alias) {
    aliases = ctx.$aliases = Object.create(ctx.$aliases);
    aliases[alias] = ctx.$depth;
  }
  if (name) ctx.$depth++;
  if (index != null) {
    ctx.$i = ctx.$i.concat(index);
    isArray = true;
  }
  if (isArray && ctx.$paths[0]) ctx.$paths[0] += '.$#';
  return ctx;
};

partialFn = function(name, type, alias, render) {
  return function(ctx, model, triggerPath, triggerId, value, index, useValue) {
    var i, indices, item, out, _ctx, _len;
    if (!useValue) value = name ? dataValue(ctx, model, name) : true;
    if (Array.isArray(value)) {
      if (type === '#') {
        ctx = extendCtx(ctx, null, name, alias, null, true);
        out = '';
        indices = ctx.$i;
        for (i = 0, _len = value.length; i < _len; i++) {
          item = value[i];
          _ctx = extend(ctx, item);
          _ctx.$i = indices.concat(i);
          out += render(_ctx, model, triggerPath);
        }
        return out;
      } else {
        if (!value.length) {
          ctx = extendCtx(ctx, value, name, alias, index);
          return render(ctx, model, triggerPath);
        }
      }
    } else {
      if ((type === '#' ? value : !value)) {
        ctx = extendCtx(ctx, value, name, alias, index);
        return render(ctx, model, triggerPath);
      }
    }
  };
};

textFn = function(name, escape) {
  return function(ctx, model) {
    var text;
    text = dataValue(ctx, model, name);
    text = text != null ? text.toString() : '';
    if (escape) {
      return escape(text);
    } else {
      return text;
    }
  };
};

blockFn = function(view, queue) {
  var alias, block, escaped, events, name, render, stack, type;
  if (!queue) return;
  stack = queue.stack, events = queue.events, block = queue.block;
  name = block.name, escaped = block.escaped, type = block.type, alias = block.alias;
  render = renderer(view, reduceStack(stack), events);
  return partialFn(name, type, alias, render);
};

pushVarFns = function(view, stack, fn, fn2, name, escaped) {
  if (fn) {
    pushChars(stack, fn);
    return pushChars(stack, fn2);
  } else {
    return pushChars(stack, textFn(name, escaped && htmlEscape));
  }
};

pushVar = function(view, stack, events, pre, post, match, fn, fn2) {
  var addEvent, alias, attrs, bound, escaped, last, name, partial, wrap;
  name = match.name, escaped = match.escaped, bound = match.bound, alias = match.alias, partial = match.partial;
  if (partial) fn = partialFn(name, '#', alias, view._find(partial));
  if (bound) {
    last = stack[stack.length - 1];
    if (wrap = pre || !last || (last[0] !== 'start') || wrapPost(post)) {
      stack.push(last = ['marker', '', {}]);
    }
    addId(view, attrs = last[2]);
    if ('contenteditable' in attrs) {
      addDomEvent(events, attrs, name, 'input', 'html');
    }
    addEvent = function(method, fn) {
      return bindEventsById(events, name, fn, attrs, method, !fn && escaped);
    };
    addEvent('html', fn);
    if (fn2) addEvent('append', fn2);
  }
  pushVarFns(view, stack, fn, fn2, name, escaped);
  if (wrap) {
    return stack.push([
      'marker', '$', {
        id: function() {
          return attrs._id;
        }
      }
    ]);
  }
};

pushVarString = function(view, stack, events, pre, post, match, fn, fn2) {
  var alias, bindOnce, bound, name, partial;
  name = match.name, bound = match.bound, alias = match.alias, partial = match.partial;
  if (partial) fn = partialFn(name, '#', alias, view._find(partial + '$s'));
  bindOnce = function(ctx) {
    ctx.$onBind(events, name);
    return bindOnce = empty;
  };
  if (bound) {
    events.push(function(ctx) {
      return bindOnce(ctx);
    });
  }
  return pushVarFns(view, stack, fn, fn2, name);
};

parse = null;

forAttr = function(view, stack, events, tagName, attrs, attr, value) {
  var anyOut, anyParser, bool, bound, del, elOut, elParser, fn, invert, match, method, name, parser, partial, post, pre, render;
  if (match = extractPlaceholder(value)) {
    pre = match.pre, post = match.post, name = match.name, bound = match.bound, partial = match.partial;
    invert = /^\s*!\s*$/.test(pre);
    if ((pre && !invert) || post || partial) {
      addId(view, attrs);
      render = parse(view, value, true, function(events, name) {
        return bindEventsByIdString(events, name, render, attrs, 'attr', attr);
      });
      attrs[attr] = function(ctx, model) {
        return attrEscape(render(ctx, model));
      };
      return;
    }
    if (parser = parsePlaceholder[attr]) {
      if (anyParser = parser['*']) anyOut = anyParser(events, attrs, name, invert);
      if (elParser = parser[tagName]) {
        elOut = elParser(events, attrs, name, invert);
      }
    }
    anyOut || (anyOut = {});
    elOut || (elOut = {});
    method = elOut.method || anyOut.method || 'attr';
    bool = elOut.bool || anyOut.bool;
    del = elOut.del || anyOut.del;
    if (bound) {
      addId(view, attrs);
      if (invert) fn = '$inv';
      bindEventsById(events, name, fn, attrs, method, attr);
    }
    if (del) {
      delete attrs[attr];
    } else {
      attrs[attr] = bool ? {
        bool: function(ctx, model) {
          if (!dataValue(ctx, model, name) !== !invert) {
            return ' ' + attr;
          } else {
            return '';
          }
        }
      } : textFn(name, attrEscape);
    }
  }
  if (!(parser = parseAttr[attr])) return;
  if (anyParser = parser['*']) anyOut = anyParser(events, attrs, value);
  if (elParser = parser[tagName]) elOut = elParser(events, attrs, value);
  anyOut || (anyOut = {});
  elOut || (elOut = {});
  if (elOut.addId || anyOut.addId) return addId(view, attrs);
};

parse = function(view, template, isString, onBind) {
  var chars, end, events, push, queues, stack, start;
  queues = [
    {
      stack: stack = [],
      events: events = []
    }
  ];
  queues.last = function() {
    return queues[queues.length - 1];
  };
  if (isString) {
    push = pushVarString;
    if (onBind) {
      pushChars(stack, function(ctx) {
        ctx.$onBind = onBind;
        ctx.$stringCtx = ctx;
        return '';
      });
    }
  } else {
    push = pushVar;
  }
  start = function(tag, tagName, attrs) {
    var attr, out, parser, value;
    if (parser = parseElement[tagName]) {
      out = parser(events, attrs) || {};
      if (out.addId) addId(view, attrs);
    }
    for (attr in attrs) {
      value = attrs[attr];
      if (attr === 'style') continue;
      forAttr(view, stack, events, tagName, attrs, attr, value);
    }
    if ('style' in attrs) {
      forAttr(view, stack, events, tagName, attrs, 'style', attrs.style);
    }
    return stack.push(['start', tagName, attrs]);
  };
  chars = function(text, literal) {
    var match, post, pre;
    if (literal || !(match = extractPlaceholder(text))) {
      pushChars(stack, trim(text));
      return;
    }
    pre = match.pre, post = match.post;
    if (isString) pre = unescapeEntities(pre);
    pushChars(stack, pre);
    parseMatch(view, match, queues, {
      onStart: function(queue) {
        var block;
        return stack = queue.stack, events = queue.events, block = queue.block, queue;
      },
      onEnd: function(queue) {
        var fn, fn2;
        fn = blockFn(view, queue);
        fn2 = blockFn(view, queue.closed);
        return push(view, stack, events, pre, post, queue.block, fn, fn2);
      },
      onVar: function(match) {
        return push(view, stack, events, pre, post, match);
      }
    });
    if (post) return chars(post);
  };
  end = function(tag, tagName) {
    return stack.push(['end', tagName]);
  };
  if (isString) {
    chars(template);
  } else {
    parseHtml(template, {
      start: start,
      chars: chars,
      end: end
    });
  }
  return renderer(view, reduceStack(stack), events);
};
