var EventDispatcher, PathMap, deleteMapItems, incrementMapItems, insert, refIndex, remove;
var __slice = Array.prototype.slice;

EventDispatcher = require('./EventDispatcher');

PathMap = function() {
  this.clear();
};

PathMap.prototype = {
  clear: function() {
    this.count = 0;
    this.ids = {};
    this.paths = {};
    return this.arrays = {};
  },
  id: function(path) {
    var id;
    return this.ids[path] || (this.paths[id = ++this.count] = path, this.indexArray(path, id), this.ids[path] = id);
  },
  indexArray: function(path, id) {
    var arr, index, match, name, remainder, set;
    if (match = /^(.+)\.(\d+)(\..+|$)/.exec(path)) {
      name = match[1];
      index = +match[2];
      remainder = match[3];
      arr = this.arrays[name] || (this.arrays[name] = []);
      set = arr[index] || (arr[index] = {});
      return set[id] = remainder;
    }
  },
  init: function(count, ids) {
    var id, path, paths, _ref, _results;
    this.count = count;
    this.ids = ids;
    paths = this.paths;
    _ref = this.ids;
    _results = [];
    for (path in _ref) {
      id = _ref[path];
      paths[id] = path;
      _results.push(this.indexArray(path, id));
    }
    return _results;
  }
};

refIndex = function(obj) {
  if (typeof obj === 'object') {
    return obj.index;
  } else {
    return +obj;
  }
};

incrementMapItems = function(pathMap, path, map, start, end, byNum) {
  var i, id, ids, itemPath, remainder, _results;
  _results = [];
  for (i = start; start <= end ? i <= end : i >= end; start <= end ? i++ : i--) {
    if (!(ids = map[i])) continue;
    _results.push((function() {
      var _results2;
      _results2 = [];
      for (id in ids) {
        remainder = ids[id];
        itemPath = path + '.' + (i + byNum) + remainder;
        pathMap.paths[id] = itemPath;
        _results2.push(pathMap.ids[itemPath] = +id);
      }
      return _results2;
    })());
  }
  return _results;
};

deleteMapItems = function(pathMap, map, start, end, last) {
  var i, id, ids, itemPath, _results;
  _results = [];
  for (i = start; start <= last ? i <= last : i >= last; start <= last ? i++ : i--) {
    if (!(ids = map[i])) continue;
    _results.push((function() {
      var _results2;
      _results2 = [];
      for (id in ids) {
        itemPath = pathMap.paths[id];
        delete pathMap.ids[itemPath];
        if (i >= end) continue;
        _results2.push(delete pathMap.paths[id]);
      }
      return _results2;
    })());
  }
  return _results;
};

insert = function(events, pathMap, path, start, values, local, options) {
  var end, howMany, i, id, last, map, value, _len;
  start = refIndex(start);
  if (map = pathMap.arrays[path]) {
    howMany = values.length;
    end = start + howMany;
    last = map.length - 1;
    deleteMapItems(pathMap, map, start, end, last);
    incrementMapItems(pathMap, path, map, start, last, howMany);
    while (howMany--) {
      map.splice(start, 0, {});
    }
  }
  id = pathMap.id(path);
  for (i = 0, _len = values.length; i < _len; i++) {
    value = values[i];
    events.trigger(id, [start + i, value], 'insert', local, options);
  }
};

remove = function(events, pathMap, path, start, howMany, local, options) {
  var end, id, index, last, map;
  start = refIndex(start);
  end = start + howMany;
  if (map = pathMap.arrays[path]) {
    last = map.length - 1;
    deleteMapItems(pathMap, map, start, end, last);
    if (!(end > last)) incrementMapItems(pathMap, path, map, end, last, -howMany);
    map.splice(start, howMany);
  }
  id = pathMap.id(path);
  for (index = start; start <= end ? index < end : index > end; start <= end ? index++ : index--) {
    events.trigger(id, index, 'remove', local, options);
  }
};

exports.init = function(model, dom, view) {
  var event, events, pathMap, _fn, _i, _len, _ref;
  pathMap = model.__pathMap = new PathMap;
  if (dom) {
    events = model.__events = new EventDispatcher({
      onBind: function(name, listener) {
        return pathMap.id(name);
      },
      onTrigger: function(name, listener, value, type, local, options) {
        var id, index, method, noRender, partial, path, property, triggerId, _ref, _ref2;
        id = listener[0], method = listener[1], property = listener[2];
        partial = listener.fn;
        path = pathMap.paths[name];
        if (method === 'propPolite' && local) method = 'prop';
        if (partial === '$inv') {
          value = !value;
        } else if (partial) {
          triggerId = id;
          if (method === 'html' && type) {
            method = type;
            if (type === 'append') {
              path += '.' + (index = model.get(path).length - 1);
              triggerId = null;
            } else if (type === 'insert') {
              _ref = value, index = _ref[0], value = _ref[1];
              path += '.' + index;
              triggerId = null;
            } else if (type === 'remove') {
              noRender = true;
            } else if (type === 'move') {
              noRender = true;
              _ref2 = value, value = _ref2[0], property = _ref2[1];
            }
          }
          if (!noRender) {
            value = partial(listener.ctx, model, path, triggerId, value, index, true);
            if (value == null) value = '';
          }
        }
        return dom.update(id, method, options && options.ignore, value, property, index);
      }
    });
  } else {
    events = model.__events = new EventDispatcher;
  }
  if (!dom) return model;
  model.on('set', function(_arg, local, options) {
    var path, value;
    path = _arg[0], value = _arg[1];
    return events.trigger(pathMap.id(path), value, 'html', local, options);
  });
  model.on('del', function(_arg, local, options) {
    var path;
    path = _arg[0];
    return events.trigger(pathMap.id(path), void 0, 'html', local, options);
  });
  model.on('push', function(_arg, local, options) {
    var id, path, value, values, _i, _len;
    path = _arg[0], values = 2 <= _arg.length ? __slice.call(_arg, 1) : [];
    id = pathMap.id(path);
    for (_i = 0, _len = values.length; _i < _len; _i++) {
      value = values[_i];
      events.trigger(id, value, 'append', local, options);
    }
  });
  model.on('move', function(_arg, local, options) {
    var from, item, len, map, path, to;
    path = _arg[0], from = _arg[1], to = _arg[2];
    len = model.get(path).length;
    from = refIndex(from);
    to = refIndex(to);
    if (from < 0) from += len;
    if (to < 0) to += len;
    if (from === to) return;
    if (map = pathMap.arrays[path]) {
      incrementMapItems(pathMap, path, map, from, from, to - from);
      if (from > to) {
        incrementMapItems(pathMap, path, map, to, from - 1, 1);
      } else {
        incrementMapItems(pathMap, path, map, from + 1, to, -1);
      }
      item = map.splice(from, 1)[0];
      map.splice(to, 0, item);
    }
    return events.trigger(pathMap.id(path), [from, to], 'move', local, options);
  });
  model.on('unshift', function(_arg, local, options) {
    var path, values;
    path = _arg[0], values = 2 <= _arg.length ? __slice.call(_arg, 1) : [];
    return insert(events, pathMap, path, 0, values, local, options);
  });
  model.on('insertBefore', function(_arg, local, options) {
    var index, path, value;
    path = _arg[0], index = _arg[1], value = _arg[2];
    return insert(events, pathMap, path, index, [value], local, options);
  });
  model.on('insertAfter', function(_arg, local, options) {
    var index, path, value;
    path = _arg[0], index = _arg[1], value = _arg[2];
    return insert(events, pathMap, path, index + 1, [value], local, options);
  });
  model.on('remove', function(_arg, local, options) {
    var howMany, path, start;
    path = _arg[0], start = _arg[1], howMany = _arg[2];
    return remove(events, pathMap, path, start, howMany, local, options);
  });
  model.on('pop', function(_arg, local, options) {
    var path;
    path = _arg[0];
    return remove(events, pathMap, path, model.get(path).length, 1, local, options);
  });
  model.on('shift', function(_arg, local, options) {
    var path;
    path = _arg[0];
    return remove(events, pathMap, path, 0, 1, local, options);
  });
  model.on('splice', function(_arg, local, options) {
    var howMany, path, start, values;
    path = _arg[0], start = _arg[1], howMany = _arg[2], values = 4 <= _arg.length ? __slice.call(_arg, 3) : [];
    remove(events, pathMap, path, start, howMany, local, options);
    return insert(events, pathMap, path, index, values, local, options);
  });
  _ref = ['connected', 'canConnect'];
  _fn = function(event) {
    return model.on(event, function(value) {
      return events.trigger(pathMap.id(event), value);
    });
  };
  for (_i = 0, _len = _ref.length; _i < _len; _i++) {
    event = _ref[_i];
    _fn(event);
  }
  return model;
};
