var addConditionalStyle, addDomEvent, distribute, modelPath, replaceIndex, splitBind, unaliasName;

replaceIndex = function(ctx, path, noReplace) {
  var i, indices;
  if (noReplace || !(indices = ctx.$i) || !path) return path;
  i = 0;
  return path.replace(/\$#/g, function() {
    return indices[i++];
  });
};

addConditionalStyle = function(attrs, name, invert, styleText) {
  var cat, style, type;
  type = invert ? '#' : '^';
  cat = "{{" + type + name + "}}" + styleText + "{{/}}";
  return attrs.style = (style = attrs.style) ? "" + style + ";" + cat : cat;
};

splitBind = function(value) {
  var delay, name, out, pair, pairs, _i, _len, _ref, _ref2;
  pairs = value.replace(/\s/g, '').split(',');
  out = {};
  for (_i = 0, _len = pairs.length; _i < _len; _i++) {
    pair = pairs[_i];
    _ref = pair.split(':'), name = _ref[0], value = _ref[1];
    _ref2 = name.split('/'), name = _ref2[0], delay = _ref2[1];
    out[name] = {
      value: value,
      delay: delay
    };
  }
  return out;
};

unaliasName = function(ctx, name) {
  var aliasName, i, offset, remainder;
  if (name.charAt(0) !== ':') return name;
  i = name.indexOf('.');
  aliasName = name.substring(1, i);
  remainder = name.substr(i + 1);
  offset = ctx.$depth - ctx.$aliases[aliasName];
  if (offset !== offset) throw new Error("Can't find alias for " + name);
  return Array(offset + 1).join('.') + remainder;
};

module.exports = {
  modelPath: modelPath = function(ctx, name, noReplace) {
    var i, paths;
    if (name in ctx) return null;
    name = unaliasName(ctx, name);
    if (name.charAt(0) === '.' && (paths = ctx.$paths)) {
      if (name === '.') return replaceIndex(ctx, paths[0], noReplace);
      i = /^\.+/.exec(name)[0].length - 1;
      return replaceIndex(ctx, paths[i], noReplace) + name.substr(i);
    }
    return name.replace(/\[([^\]]+)\]/g, function(match, name) {
      return ctx[name];
    });
  },
  addDomEvent: addDomEvent = function(events, attrs, name, _eventNames, getMethod, property, invert) {
    var delay, eventName, eventNames, i, isArray, _len, _ref;
    if (isArray = Array.isArray(_eventNames)) {
      eventNames = [];
      for (i = 0, _len = _eventNames.length; i < _len; i++) {
        eventName = _eventNames[i];
        eventNames[i] = eventName.split('/');
      }
    } else {
      _ref = _eventNames.split('/'), eventName = _ref[0], delay = _ref[1];
    }
    return events.push(function(ctx, modelEvents, domEvents) {
      var id, path, _i, _len2, _ref2;
      path = modelPath(ctx, name);
      id = attrs._id || attrs.id;
      if (isArray) {
        for (_i = 0, _len2 = eventNames.length; _i < _len2; _i++) {
          _ref2 = eventNames[_i], eventName = _ref2[0], delay = _ref2[1];
          domEvents.bind(eventName, [path, id, getMethod, property, delay, invert]);
        }
      } else {
        domEvents.bind(eventName, [path, id, getMethod, property, delay, invert]);
      }
    });
  },
  distribute: distribute = function(events, attrs, eventName) {
    return events.push(function(ctx, modelEvents, domEvents) {
      return domEvents.bind(eventName, ['$dist', attrs._id || attrs.id]);
    });
  },
  parsePlaceholder: {
    'value': {
      input: function(events, attrs, name) {
        var eventNames;
        if ('x-blur' in attrs) {
          delete attrs['x-blur'];
          eventNames = 'change';
        } else {
          eventNames = 'input';
        }
        addDomEvent(events, attrs, name, eventNames, 'prop', 'value');
        return {
          method: 'propPolite'
        };
      }
    },
    'checked': {
      '*': function(events,  attrs, name, invert) {
        addDomEvent(events, attrs, name, 'change', 'prop', 'checked', invert);
        return {
          method: 'prop',
          bool: true
        };
      }
    },
    'selected': {
      '*': function(events, attrs, name, invert) {
        addDomEvent(events, attrs, name, 'change', 'prop', 'selected', invert);
        return {
          method: 'prop',
          bool: true
        };
      }
    },
    'disabled': {
      '*': function() {
        return {
          method: 'prop',
          bool: true
        };
      }
    },
    'x-visible': {
      '*': function(events, attrs, name, invert) {
        addConditionalStyle(attrs, name, invert, 'visibility:hidden');
        return {
          method: 'visible',
          del: true
        };
      }
    },
    'x-displayed': {
      '*': function(events, attrs, name, invert) {
        addConditionalStyle(attrs, name, invert, 'display:none');
        return {
          method: 'displayed',
          del: true
        };
      }
    }
  },
  parseElement: {
    'select': function(events, attrs) {
      distribute(events, attrs, 'change');
      return {
        addId: true
      };
    }
  },
  parseAttr: {
    'x-bind': {
      '*': function(events, attrs, value) {
        var name, obj, _fn, _ref;
        delete attrs['x-bind'];
        _ref = splitBind(value);
        _fn = function(name) {
          var delay;
          value = obj.value, delay = obj.delay;
          return events.push(function(ctx, modelEvents, domEvents) {
            return domEvents.bind(name, [value, attrs._id || attrs.id, delay]);
          });
        };
        for (name in _ref) {
          obj = _ref[name];
          _fn(name);
        }
        return {
          addId: true
        };
      },
      a: function(events, attrs, value) {
        var obj;
        obj = splitBind(value);
        if ('click' in obj && !('href' in attrs)) {
          attrs.href = '#';
          if (!('onclick' in attrs)) return attrs.onclick = 'return false';
        }
      },
      form: function(events, attrs, value) {
        var obj;
        obj = splitBind(value);
        if ('submit' in obj) {
          if (!('onsubmit' in attrs)) return attrs.onsubmit = 'return false';
        }
      }
    }
  }
};
